import csv
import statistics
import os

# File parameters
number_runs = 5
prefix_files = 'input/results_'
suffix_files = ".csv"
delimiter = ","

# Experiment and analysis parameters
default_keep = "Time"
default_ignore_keep = []  # to force column check if needed
value_for_TO = 120
heuristic_none = "no-"
output_file = "output_result_time.csv"
input_has_merges = "output_result_has_merges.csv"


### Charge data
def charge_values(file, keep=default_keep, ignore_keep=default_ignore_keep):
    """
    Create dict from a result csv file
    :param file: result csv file
    :return: Dict { "model" : { "property" : { 'RVM': .., 'ROM' : ... } } }
    """
    dict = {}
    with open(file, "r") as csv_file:
        csv_reader = csv.DictReader(csv_file, delimiter=delimiter)
        for lines in csv_reader:
            model = lines["Model"]
            property = lines["Property"]

            try:
                dict[model][property] = {}
            except KeyError:
                dict[model] = {}
                dict[model][property] = {}

            for key, value in lines.items():
                if key in ["Model", "Property"]:
                    pass
                if keep in key:
                    bol = True
                    for k in ignore_keep:
                        if k in key:
                            bol = False
                    if bol:
                        new_key = key.replace(keep, "").replace(" ", "").replace("merge", "")
                        #if new_key in ["no-", "212", "RVMr", "OQM"]:
                        new_value = float(value.replace(",", ".")) if value != "" else value_for_TO
                        dict[model][property][new_key] = new_value
    return dict


def charge_all(keep=default_keep, ignore_keep=default_ignore_keep):
    """
    Create dict { run0 : {results}, run2: {results}, ...}
    :return:
    """
    dict = {}
    for run in range(number_runs):
        index_run = run + 1
        dict["run{}".format(run)] = charge_values("{}{}{}".format(prefix_files, index_run, suffix_files), keep,
                                                  ignore_keep)
    return dict


### Has merges
def charge_has_merges():
    """
    Charge if a (model, property) performs a merge from written file
    :return: dict { model { property : True/False } }
    """
    dict = {}
    with open(input_has_merges, "r") as csv_file:
        csv_reader = csv.DictReader(csv_file, delimiter=delimiter)
        for lines in csv_reader:
            model = lines["Model"]
            property = lines["Property"]
            has_merges = lines["Has merges?"]

            if has_merges == "True":
                bool = True
            elif has_merges == "False":
                bool = False
            else:
                raise "Invalid string"

            try:
                dict[model][property] = bool
            except KeyError:
                dict[model] = {}
                dict[model][property] = bool

    return dict


### Average, etc.
def average_for_execution(dict_experiment):
    """
    Return a dict where the result for each (model, property) is shown
    :param dict_experiment: Dict of all runs {run0: ..., run2: ...}
    :return: Dict of results {model : { property : { RVM : { avg(run1, run2, .. } } }
    """
    dict = {}
    for model, t_value in dict_experiment["run0"].items():
        dict[model] = {}
        for property, tt_value in t_value.items():
            dict[model][property] = {}
            for heuristic in tt_value.keys():
                L = []
                for key in dict_experiment.keys():
                    L.append(dict_experiment[key][model][property][heuristic])
                dict[model][property][heuristic] = statistics.mean(L)
    return dict


def results(average_dict, has_merges):
    """
    Return results (average, norm. average, norm. average with/without merges) from average_dict
    :param average_dict: result of average_for_execution
    :return: { heuristic: { average: .., norm. average : ..., ...  } }
    """
    dict = {}
    first_model = list(average_dict.keys())[0]
    first_property = list(average_dict[first_model].keys())[0]
    for heuristic in average_dict[first_model][first_property].keys():
        wins = 0
        L_average = []
        L_norm_average = []
        L_average_merges = []
        L_norm_average_merges = []
        L_average_no_merges = []
        L_norm_average_no_merges = []
        # average
        for model, t_values in average_dict.items():
            for property, tt_values in t_values.items():
                # wins
                values_for_all_heuristics = average_dict[model][property].values()
                value_for_heuristic = average_dict[model][property][heuristic]
                # win if value is the smaller but not the bigger
                if value_for_heuristic == min(values_for_all_heuristics) and value_for_heuristic != max(
                        values_for_all_heuristics):
                    wins += 1

                # averages etc.
                L_average.append(average_dict[model][property][heuristic])
                try:
                    L_norm_average.append(
                    average_dict[model][property][heuristic] / average_dict[model][property][heuristic_none])
                except ZeroDivisionError:
                    pass
                if has_merges[model][property]:
                    L_average_merges.append(average_dict[model][property][heuristic])
                    try:
                        L_norm_average_merges.append(
                        average_dict[model][property][heuristic] / average_dict[model][property][heuristic_none])
                    except ZeroDivisionError:
                        pass
                else:
                    L_average_no_merges.append(average_dict[model][property][heuristic])
                    try:
                        L_norm_average_no_merges.append(
                        average_dict[model][property][heuristic] / average_dict[model][property][heuristic_none])
                    except ZeroDivisionError:
                        pass

        dict[heuristic] = {}
        dict[heuristic]["wins"] = wins
        dict[heuristic]["average"] = statistics.mean(L_average)
        dict[heuristic]["average with merge"] = statistics.mean(L_average_merges)
        dict[heuristic]["average without merge"] = statistics.mean(L_average_no_merges)
        dict[heuristic]["median"] = statistics.median(L_average)
        dict[heuristic]["norm average"] = statistics.mean(L_norm_average)
        dict[heuristic]["norm average with merge"] = statistics.mean(L_norm_average_merges)
        dict[heuristic]["norm average without merge"] = statistics.mean(L_norm_average_no_merges)
        dict[heuristic]["norm median"] = statistics.median(L_norm_average)
    return dict


def write_results(results):
    """
    Write a csv with results from output of results(..)
    :param results:
    :return: None
    """
    header = ["Statistic"] + list(results.keys())
    print(header)
    with open(output_file, 'w') as f:
        dict_writer = csv.DictWriter(f, fieldnames=header)
        dict_writer.writeheader()
        for stat in list(results[heuristic_none].keys()):
            row = {}
            row["Statistic"] = stat
            for heuristic in list(results.keys()):
                row[heuristic] = str(results[heuristic][stat]).replace(".", ",")
            dict_writer.writerow(row)


if __name__ == "__main__":
    dict = charge_all()
    avg = average_for_execution(dict)
    has_merges = charge_has_merges()
    res = results(avg, has_merges)
    print(res)
    write_results(res)
