input_time = "output_result_time.csv"
input_states = "output_result_states.csv"
output_latex = "output_latex.tex"

import csv

# define display of heuristics
heuristics = []
for algo in ["reconstruct", "onthefly"]:
    for candidates in ["visited", "queue", "ordered"]:
        for update in ["merge", "candidates"]:
            for restart in ["on", "off"]:
                heuristics.append("{}{}{}{}".format(algo[0].upper(), candidates[0].upper(), update[0].upper(),
                                      "r" if restart == "on" else ""))

heuristic_to_latex = {
    "no-": "\\nomerge",
   "212": "\\mergeTwoTwelve"
}
heuristic_to_text = {
    "no-": "Nomerge",
   "212": "M2.12"
}
for heuristic in heuristics:
    heuristic_to_latex[heuristic] = "\\merge{}".format(heuristic)
    heuristic_to_text[heuristic] = "{}".format(heuristic)

heuristic_head = ""
for heuristic, text in heuristic_to_latex.items():
    heuristic_head += "\\newcommand{" + text + "}{\\texttt{" + heuristic_to_text[heuristic] + "}}\n"

# head and foot of tex file
begin_head = " \\documentclass{article}\n" \
        "\n" \
        "\\usepackage[landscape,margin=1cm]{geometry}\n" \
        "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n" \
        "% COLORS AND TABLES\n" \
        "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n" \
        "\\usepackage[svgnames,table]{xcolor}\n" \
        "\\usepackage{rotating}\n" \
        "\\usepackage{multirow,array}\n" \
        "\\newcolumntype{M}[1]{>{\\centering\\arraybackslash}m{#1}}\n" \
        "% \\PassOptionsToPackage{table}{xcolor}\n" \
        "\\newcommand{\\cellHeader}[1]{\\cellcolor{blue!20}\\textbf{#1}}\n" \
        "\\newcommand{\\rowHeader}{\\rowcolor{blue!20}}\n" \
        "\n" \
        "\\newcommand{\\cellYes}{\\cellcolor{green!40}\\textbf{$\\surd$}}\n" \
        "\\newcommand{\\cellNo}{\\cellcolor{red!40}\\textbf{$\\times$}}\n" \
        "\n" \
        "% \\newcommand{\\cellBest}{\\cellcolor{green!40}}\n" \
        "\\newcommand{\\cellBest}{\\cellcolor{green!80}\\bfseries}\n" \
        "\\newcommand{\\cellOne}{\\cellcolor{green!10}}\n" \
        "\\newcommand{\\cellTwo}{\\cellcolor{green!20}}\n" \
        "\\newcommand{\\cellThree}{\\cellcolor{green!30}}\n" \
        "\\newcommand{\\cellFour}{\\cellcolor{green!40}}\n" \
        "\\newcommand{\\cellFive}{\\cellcolor{green!50}}\n" \
        "\n"

begin_document=\
    "\\begin{document}\n" \
        "\\begin{table}[tb]\n" \
        "	\\centering\n" \
        "	\\tiny\n"

begin = begin_head + heuristic_head + begin_document

end = "\\end{table}\n" \
      "\\end{document}"


rank_text = {
    1: "\\cellOne{\\cellBest{",
    2: "\\cellTwo{",
    3: "\\cellThree{",
    4: "\\cellFour{",
    5: "\\cellFive{"
}


def charge_values(file):
    """
    Create dict from a result csv file
    :param file: result csv file
    :return: Dict { stat: { heuristic: .. } }
    """
    dict = {}
    with open(file, "r") as csv_file:
        csv_reader = csv.DictReader(csv_file)
        for lines in csv_reader:
            stat = lines.pop("Statistic")
            dict[stat] = lines
    return dict


def to_number(value, decimal=2):
    v = value.replace(",", ".")
    v = round(float(v), decimal)
    return str(v)


def rank(value, list, order="acs"):
    if order == "acs":
        L = [float(to_number(k)) for k in list]
        L = sorted(L)
    elif order == "desc":
        L = [float(to_number(k)) for k in list]
        L = sorted(L, reverse=True)
    else:
        raise ("Rank function order")
    return L.index(float(to_number(value))) + 1


def display_rank(value, list, order="acs"):
    rank_v = rank(value, list, order)
    if rank_v in [k for k in range(1, len(rank_text) + 1)]:
        txt = rank_text[rank_v] + to_number(value) + "}"
        if rank_v == 1: txt += "}"
    else:
        txt = to_number(value)
    return txt


def write_latex(time, states):
    file = open(output_latex, "w")
    file.write(begin)

    # begin
    columns = ["c|l"] + ["l" for heuristic in time["average"]]
    columns = "|" + "|".join(columns) + "|"
    txt = "\\begin{tabular}{" + columns + "}\n"
    file.write(txt)

    # rowheader
    txt = "\\hline\n"
    columns = ["", ""] + [heuristic_to_latex[heuristic] for heuristic in time["average"]]
    columns = "&".join(columns)
    txt += "\\rowHeader " + columns + "\\\\\n"
    txt += "\\hline\n"
    file.write(txt)

    # Time
    wins = ["\\multirow{" + str(len(time)) + "}{*}{\\rotatebox[origin=c]{90}{Time}}"] + ["\# wins"] + [
        display_rank(time["wins"][heuristic], time["wins"].values(), "desc") for heuristic in time["average"]]
    wins = "&".join(wins) + "\\\\\n"
    file.write(wins)
    for stat in time.keys():
        if stat == "wins":
            continue
        values = [""] + [stat] + [display_rank(v, time[stat].values()) for v in time[stat].values()]
        txt = "&".join(values) + "\\\\\n"
        file.write(txt)

    file.write("\\hline\n")

    # States
    wins = ["\\multirow{" + str(len(states)) + "}{*}{\\rotatebox[origin=c]{90}{States}}"] + ["\# wins"] + [
        display_rank(states["wins"][heuristic], states["wins"].values(), "desc") for heuristic in
        states["average"]]
    wins = "&".join(wins) + "\\\\\n"
    file.write(wins)
    for stat in states.keys():
        if stat == "wins":
            continue
        values = [""] + [stat] + [display_rank(v, states[stat].values()) for v in states[stat].values()]
        txt = "&".join(values) + "\\\\\n"
        file.write(txt)

    file.write("\\hline\n")

    # end
    txt = "\\end{tabular}"
    file.write(txt)
    file.write(end)
    file.close()


if __name__ == "__main__":
    time = charge_values(input_time)
    states = charge_values(input_states)
    print(time)
    print(states)
    write_latex(time, states)
